--Created by Joshua Garrett
--Inspired by City Trial from Kirby Air Ride
--Description
    --Players have a limited time to collect power-ups.
    --After the timer runs down players respawn to play out a EMFH match with the power-ups they have collected

    Triggers = {}
    CollectionsUsed = { 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,31 }

    timer = -1
    MatchLength = 0
    Countdown = 11
    ScavengeTimer = 99999 --Timer for intro where players can acquire powerups
    CarnageTimer = 99999 --Timer for emfh
    ScavengeComplete = 0 --If ScavengeTimer has reached 0
    IntermissionComplete = 0
    IntermissionCountdown = 4
    IntermissionTimer = 90
    MonstersCleared = 0
    RegenTimer = 90
    DefaultHealth = 150
    PlayerSetupRun = 0 --used to check if player setup has been run since it cannot be run in Triggers.init and Triggers.idle runs every tick
    StatCount = 0
    PlayerCount = 0
    PolyCount = 0
    StatToCompare = 0
    MonsterCount = 0
    MonsterLimit = 36
    HealthIncrement = 10
    DamageIncrement = 1
    DodgeIncrement = 1
    InvisibilityIncrement = 1
    Combatants = {
        "green bob", -- +10 max health
        "security bob", -- +1 damage to every attack (might have to edit this per weapon we'll see)
        "blue bob", -- Seconds of invisibility on spawn (speed seems hecking jank in marathon when being modified)
        --"explodabob", --+1% to damage returned as health (Cut because of visual similarities to other bobs, too confusing to tell what powerup you were getting, will add back in if requested)
        "tiny bob"  -- +1 to "dodge" player has 1% chance to dodge incoming damage (this would need some audio cue for the attacker to know)
    }

    function Triggers.init()
        poly_init()
        monster_setup()
        table_length()
    end


    function Triggers.idle()
        if PlayerSetupRun == 0 then
        player_setup()
        stat_update()
        MatchLength = Game.time_remaining
        timer_setup()
        PlayerSetupRun = 1
        end

        if ScavengeTimer == 0 and IntermissionTimer == 0 then
            ScavengeTimer = 0
            CarnageTimer = CarnageTimer - 1
           -- Players.print("SCAVEG_TIMER IS ZERO")
        elseif ScavengeTimer > 0 then
            ScavengeTimer = ScavengeTimer - 1
            player_color()
            monster_spawning()
            overlays()
           -- Players.print("ScavengeTimer IS OVER ZERO")
        end
        if ScavengeTimer == 0 and MonstersCleared == 0 then
            monster_clearing()
            MonstersCleared = 1
          --  Players.print("MONSTERS WERE CLEARED")
        end
        if ScavengeComplete == 1 and IntermissionComplete == 0 then
            intermission()
        end
        for plr in Players() do
            --health_regen(plr)
        end
        notifications()
    end


    function poly_init()
        for poly in Polygons() do
            poly._blacklisted = 0
            PolyCount = PolyCount + 1
            if poly.area <= 2 then
                poly._blacklisted = 1
            end
            if poly.type == "platform" then
                poly._blacklisted = 1
            end
            if poly.type  == "teleporter" then
                poly._blacklisted = 1
            end
                for plrstart in PlayerStarts() do
                    if poly == plrstart.polygon then
                        poly._blacklisted = 1
                    end
                end
        end
    end


    function timer_setup()
        ScavengeTimer = MatchLength*.25
        CarnageTimer = MatchLength*.75 - IntermissionTimer
    end

    function table_length() --used to see how many monsters/powerup are being used
        for k,v in ipairs(Combatants) do
            StatCount = StatCount + 1
        end
    end


    function player_setup()
        for plr in Players() do
            plr._stats = {} --creates a table for each player to store things like max health, damage, etc
            power_up = 1
            PlayerCount = PlayerCount + 1
            plr.items["pistol"] = 0
            plr.items["flamethrower"] = 1
            plr.weapons["flamethrower"]:select()
            plr.color = "slate"
            plr.team = "slate"
            plr._max_stat = -1
            plr._first_kill = 0
            while power_up <= StatCount do
            table.insert(plr._stats, power_up, 0) --Creates an entry for each weapon with the index being the key and the level of the weapon being the value
            power_up = power_up + 1
            end
        end
    end

    function replenish_ammo()
        for plr in Players() do
            if timer % 90 == 0 then
                plr.items["flamethrower ammo"] = 1
            end
        end
    end

    function suppress_items()
        for i in Items() do
            if ScavengeComplete == 0 then
                i:delete()
            end
        end
    end

    function monster_spawning()
        spawn_chance = Game.random(6)
        for poly in Polygons() do
            if MonsterCount < MonsterLimit and spawn_chance == 0 then
                monster_to_spawn = Game.random(StatCount) + 1 --pick a new monster to spawn
                    if poly._blacklisted == 0 then
                        Monsters.new(poly.x, poly.y, 0, poly, Combatants[monster_to_spawn]) --Create the new monster and spawn it on the polygon
                        MonsterCount = MonsterCount + 1
                    end
            end
		end
	end


    function health_regen(plr)
        if plr._hit then
             if plr._timeHit == 0 then
                 plr._hit = false
             else
                 plr._timeHit = plr._timeHit - 1
             end
        end
        if not plr._hit and plr.life < plr._maxHealth and not plr.dead then
            plr:play_sound("energy refuel", 1)
            plr.life = plr.life + 1
            if plr.life >= plr._maxHealth then
                plr.life = plr._maxHealth
                plr._hit = true
                plr._timeHit = RegenTimer
            end
        end
    end

    function monster_setup()
        for m in MonsterTypes() do
            for n in MonsterClasses() do
                m.enemies["player"]=true
                m.friends["player"]=false
                if n ~= "player" then
                    m.friends[n]=true
                    m.enemies[n]=false
                end
            end
        end
    end

    function monster_clearing()
        for m in Monsters() do
            if m.type ~= "player" and ScavengeTimer == 0 then
                --m.life = 1
                m:damage(m.life + 1)
            end
        end
        for plr in Players() do
            if ScavengeTimer == 0 then
                plr:damage(plr.life + 1)
            end
        end
        --Players.print("MONSTERS CLEARED")
    end

    function Triggers.monster_killed(monster, aggressor_player, projectile)
       -- Players.print(monster.."was killed")
        if aggressor_player then
            for k,v in ipairs(Combatants) do --look through Combatants
                if monster.type == v then -- if value from the table matches the monster that was killed then
                    aggressor_player._stats[k] = aggressor_player._stats[k] + 1 -- increase the stat that corresponds to the monster killed
                    stat_update()
                end
            end
            aggressor_player._first_kill = 1 --if player has gotten a kill yet (keep player/team color from changing at match start)
        end
        if not monster.player then
            MonsterCount = MonsterCount - 1
        end
    end

    function Triggers.player_damaged(victim, aggressor_player, aggressor_monster, damage_type, damage_amount, projectile)
        victim._dodge = Game.random(99) + 1
            if victim._dodge <= victim._bonusDodge then
                victim:play_sound("ball bounce", 3)
                victim.life = victim.life + damage_amount
                if aggressor_player then
                    aggressor_player:play_sound("ball bounce", 3)
					aggressor_player:print(victim.." dodged your shot")
                end
            else
                if victim._dodge > victim._bonusDodge then
                    victim._hit = true
                        if victim.life < victim._maxHealth then
                            victim._timeHit = RegenTimer
                        end
                        if aggressor_player then
                            damage_amount = damage_amount + aggressor_player._bonusDamage
                        end
                end
            end

    end

    function Triggers.player_killed(player, aggressor_player, action, projectile)
        if ScavengeTimer == 0 then
            if aggressor_player then
                if aggressor_player ~= player
                    aggressor_player.points = aggressor_player.points + 1
                end
            end
        end
    end

    function Triggers.player_revived(player)
        player.invisibility_duration = player._invisOnSpawn
            if player and ScavengeTimer > 0 then
                for k,v in ipairs(player._stats) do
                    if player._stats[k] <= 3 then
                        player._stats[k] = 0
                    else
                        player._stats[k] = player._stats[k] - 3
                    end
                end
                player.items["pistol"] = 0
                player.items["flamethrower"] = 1
                player.weapons["flamethrower"]:select()
            end
        stat_update()
    end

    function stat_update()
        for plr in Players() do
            plr._bonusHealth = plr._stats[1]*HealthIncrement
            plr._maxHealth = plr._bonusHealth + DefaultHealth
            plr._bonusDamage = plr._stats[2]*DamageIncrement
            plr._invisOnSpawn = (plr._stats[3]*30)*InvisibilityIncrement
            plr._bonusDodge = plr._stats[4]*DodgeIncrement
        end
    end

    function player_color()
        for plr in Players() do
            for k,v in ipairs(plr._stats) do
                if v > plr._max_stat then
                    plr._max_stat = v
                    plr._color_value = k
                end
            end
        if plr._first_kill == 1 then
            if plr._color_value == 1 then
                plr.color = "red"
                plr.team = "red"
            elseif plr._color_value == 2 then
                plr.color = "yellow"
                plr.team = "yellow"
            elseif plr._color_value == 3 then
                plr.color = "blue"
                plr.color = "blue"
            elseif plr._color_value == 4 then
                plr.color = "green"
                plr.team = "green"
            end
        end
        end
    end

    function intermission()
        if IntermissionTimer % 30 == 0 and IntermissionComplete == 0 then
            IntermissionCountdown = IntermissionCountdown - 1
            Players.print("Intermission..."..IntermissionCountdown)
        end
        if ScavengeTimer == 0 and IntermissionTimer > 0 then
            IntermissionTimer = IntermissionTimer - 1
        elseif IntermissionTimer == 0  then
            for plr in Players() do
                plr.action_flags.action_trigger = true
            end
            IntermissionComplete = 1
        end
        if ScavengeTimer == 0 and MonstersCleared == 1 and IntermissionComplete == 0 then
            for plr in Players() do
                plr.action_flags.action_trigger = false
            end
        end
    end


    function overlays()
        for plr in Players() do
            --Max Health
            plr.overlays[0].color = "red"
            plr.overlays[0].text = "Max Health "..plr._maxHealth
            --Damage Bonus
            plr.overlays[1].color = "yellow"
            plr.overlays[1].text = "Damage + "..plr._bonusDamage
            --Invisibility on Spawn
            plr.overlays[2].color = "blue"
            plr.overlays[2].text = "Invis "..plr._invisOnSpawn
            --Bonus Dodge Chance
            plr.overlays[3].color = "green"
            plr.overlays[3].text = "Dodge Chance "..plr._bonusDodge
        end
    end

    function notifications()
        --Players.print("carnage_time = "..CarnageTimer)
        --Players.print("ScavengeTimer ="..ScavengeTimer)
        if CarnageTimer == 330 then
            Countdown = 11
            --Players.print("CarnageTimer = 330")
        elseif CarnageTimer <= 300 then
            --Players.print("carnge_timer < 300")
            if CarnageTimer % 30 == 0 then
                Countdown = Countdown - 1
                Players.print(Countdown)
            end
        elseif ScavengeTimer <= 300 and ScavengeComplete ~= 1 then
            --Players.print("ScavengeTimer")
            if ScavengeTimer % 30 == 0 and Countdown > 0 then
            Countdown = Countdown - 1
            Players.print(Countdown)
            end
            if Countdown == 0 then
            ScavengeComplete = 1
            end
        end
    end
